﻿using BankContracts.BindingModels;
using BankContracts.BusinessLogicsContracts;
using BankContracts.SearchModels;
using BankContracts.StoragesContracts;
using BankContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BankBusinessLogic.BusinessLogics
{
    public class DealLogic : IDealLogic
    {
        private readonly ILogger _logger;
        private readonly IDealStorage _dealStorage;

        public DealLogic(ILogger<DealLogic> logger, IDealStorage dealStorage)
        {
            _logger = logger;
            _dealStorage = dealStorage;
        }

        public List<DealViewModel>? ReadList(DealSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{Id}", model?.Id);
            var list = model == null ? _dealStorage.GetFullList() : _dealStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public DealViewModel? ReadElement(DealSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{Id}", model.Id);
            var element = _dealStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(DealBindingModel model)
        {
            CheckModel(model);
            if (_dealStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(DealBindingModel model)
        {
            CheckModel(model);
            if (_dealStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(DealBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_dealStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(DealBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.ClientId < 0)
            {
                throw new InvalidOperationException("Неверный ID клиента");
            }
            if (model.OperatorId < 0)
            {
                throw new InvalidOperationException("Неверный ID оператора");
            }
            _logger.LogInformation("Deal. DealDate:{DealDate}. ClientId:{ClientId}. OperatorId:{OperatorId}. Id:{Id}", model.DealDate, model.ClientId, model.OperatorId, model.Id);
        }
    }
}
